use serde::Deserialize;

use crate::image::Pixel;

#[derive(Deserialize, Debug)]
struct ColorEntry {
    color: Pixel,
    symbol: String,
}

#[derive(Deserialize, Debug)]
pub struct Mapping(Vec<ColorEntry>);

impl Mapping {
    pub fn load(path: impl AsRef<std::path::Path>) -> Result<Mapping, Box<dyn std::error::Error>> {
        Ok(serde_yaml::from_reader(std::fs::File::open(path)?)?)
    }

    pub fn lookup(&self, color: Pixel) -> &str {
        let found = self.0.iter().find(|entry| entry.color == color);
        match found {
            Some(entry) => &entry.symbol,
            None => panic!("Unable to find entry for {:?}", color),
        }
    }
}

#[derive(Deserialize)]
pub struct Config {
    pub grid_every: u32,
    pub line_weight: f64,
    pub major_line_weight: f64,
    pub grid_size: f64,
    pub font: String,
}

impl Config {
    pub fn scale(&self, n: u32) -> f64 {
        n as f64 * self.grid_size
    }
}

impl Default for Config {
    fn default() -> Config {
        Config {
            grid_every: 10,
            line_weight: 1.0,
            major_line_weight: 3.0,
            grid_size: 24.0,
            font: "Fira Sans 12".to_string(),
        }
    }
}
