mod data;
mod draw;
mod image;
mod opts;

use clap::Parser;

use crate::data::{Config, Mapping};
use crate::draw::Pattern;
use crate::image::Image;
use crate::opts::Options;

fn main() -> Result<(), Box<dyn std::error::Error>> {
    let opts = Options::parse();

    let image = Image::load(opts.image)?;
    let mapping = Mapping::load(opts.mapping)?;

    let config = Config {
        grid_every: opts.grid,
        line_weight: opts.line_weight,
        major_line_weight: opts.major_line_weight,
        grid_size: opts.size,
        font: opts.font.clone(),
    };

    let surf = cairo::SvgSurface::new(
        config.scale(image.width),
        config.scale(image.height),
        Some(opts.output.unwrap_or_else(|| "output.svg".to_string())),
    )?;

    Pattern {
        image,
        config,
        mapping,
    }
    .draw(cairo::Context::new(&surf)?)?;

    surf.finish();

    Ok(())
}
